@@ -247,13 +247,11 @@ class Tag:
         yield self_rendered
         if not body:
             return
-        if not isinstance(body, dict):
+        if isinstance(body, str):
             code = u.compile_template(body)
             if code is not None:
-                yield eval(code, None, self_ctx)
-            else:
-                # no code found in body
-                yield body
+                body = eval(code, None, self_ctx)
+            yield self.format_text_body(body)
             return
 
         yield u.START_BODY
@@ -262,6 +260,13 @@ class Tag:
             yield from ch.render(u, ch_ctx, ch_body)
         yield u.END_BODY
 
+    def format_text_body(self, body: str):
+        """Return formatted/escaped body.
+
+        NOTE: This hook is only invoked if the body is a `string` (i.e. text node).
+        """
+        return body
+
     def __repr__(self):
         nm = self.tag_name or self.__class__.__name__
         return f'<{nm}({str(self.attrs)})>'
@@ -351,13 +356,36 @@ class SlotTemplate(MetaTag):
         return v
 
 
-class Component(MetaTag):
+class ComponentMeta(type):
+    def __init__(cls, name, bases, dct):
+        super().__init__(name, bases, dct)
+        if name == 'Component':
+            return
+        template_processed = cls.__dict__.get('_template_processed', False)
+        if not template_processed:
+            cls._template_processed = True
+            if cls.template is None:
+                template = cls.template = cls.template_factory(cls)
+            else:
+                template = cls.template
 
-    template: Union[str, dict]
+            # if we have root - pass down attrs
+            cls.has_root = (
+                isinstance(template, dict)
+                and len(template) == 1
+                and not isinstance([*template][0], Slot)
+            )
+
+
+class Component(MetaTag, metaclass=ComponentMeta):
+
+    template: Union[str, dict] = None
+    template_factory: Callable
+    has_root = False
+    _template_processed = False
 
     # instance attrs
     props: Union[list, dict]
-    has_root: bool
     assign_attrs: Dict
 
     @overload
@@ -374,12 +402,6 @@ class Component(MetaTag):
     def __init__(self, **attrs):
         super().__init__(**attrs)
         self.slots = set()
-        # if we have root - pass down attrs
-        self.has_root = (
-            isinstance(self.template, dict)
-            and len(self.template) == 1
-            and not isinstance([*self.template][0], Slot)
-        )
 
     def _process_attrs(self, attrs: dict):
         if isinstance(self.props, list):
@@ -673,9 +695,10 @@ class HTMLPrinter:
             self._print_with_indent(it)
             self.end_body()
         else:
-            close_tag = self.stack and self.stack[-1]
+            stack = self.stack
+            close_tag = stack[-1] if len(stack) else None
             if isinstance(close_tag, str):
-                self.stack.pop()
+                stack.pop()
                 self._print(close_tag)
             if not self.debug and it.tag_class.is_meta_tag:
                 tag_def = None
@@ -690,7 +713,7 @@ class HTMLPrinter:
                 end_tag_def, close_tag = ['', f'</{tag_name}>'] if it.is_body_allowed else [' /', '']
                 tag_def = f'<{tag_name}{sep}{attrs}{end_tag_def}>'
             self._print_with_indent(tag_def)
-            self.stack.append(close_tag)
+            stack.append(close_tag)
         self.prev_tag = it
 
 
